package cn.estsh.i3plus.ext.mes.apiservice.controller.busi.bu;

import cn.estsh.i3plus.ext.mes.api.busi.bu.IMesSkillsManagementBenchmarkService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesSkillsManagementBenchmark;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SessionUser;
import cn.estsh.impp.framework.base.controller.MesBaseController;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping(CommonConstWords.BASE_URL_MES+"/skills-management-benchmark")
@Api(tags = "技能管理基准")
public class MesSkillsManagementBenchmarkController extends MesBaseController {

    @Autowired
    private IMesSkillsManagementBenchmarkService skillsManagementBenchmarkService;

    @GetMapping(value = "/query-list-page")
    @ApiOperation(value = "技能管理基准分页查询")
    public ResultBean getSkillsManagementBenchmarkPage(MesSkillsManagementBenchmark skillsManagementBenchmark, Pager pager) {
        String organizeCode = AuthUtil.getOrganize().getOrganizeCode();
        try {
            //请求参数非空
            if (null == skillsManagementBenchmark || null == pager) {
                return ResultBean.fail("请求参数不完整");
            }

            ResultBean resultBean = new ResultBean(true);
            return ResultBean.success("技能管理基准分页查询")
                    .setListPager(skillsManagementBenchmarkService.getSkillsManagementBenchmarkPage(skillsManagementBenchmark, pager, organizeCode))
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException imppException) {

            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/insert")
    @ApiOperation(value = "新增技能管理基准")
    public ResultBean insertSkillsManagementBenchmark(MesSkillsManagementBenchmark skillsManagementBenchmark) {
        String userName = AuthUtil.getSessionUser().getUserName();
        SessionUser sessionUser = AuthUtil.getSessionUser();
        String organizeCode = AuthUtil.getOrganize().getOrganizeCode();
        try {
            // 属性验证
            /*ValidatorBean.beginValid(mesWorkOrder)
                    .notNull("id", mesWorkOrder.getId());*/
            skillsManagementBenchmarkService.insertSkillsManagementBenchmark(skillsManagementBenchmark, organizeCode, userName);
            return new ResultBean(true).setMsg("新增成功");
        } catch (ImppBusiException imppException) {

            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PutMapping(value = "/update")
    @ApiOperation(value = "更新技能管理基准")
    public ResultBean updateSkillsManagementBenchmark(MesSkillsManagementBenchmark skillsManagementBenchmark) {
        String userName = AuthUtil.getSessionUser().getUserName();
        String organizeCode = AuthUtil.getOrganize().getOrganizeCode();
        try {
            // 属性验证
            ValidatorBean.beginValid(skillsManagementBenchmark)
                    .notNull("id", skillsManagementBenchmark.getId());
            skillsManagementBenchmarkService.updateSkillsManagementBenchmark(skillsManagementBenchmark, organizeCode, userName);
            return new ResultBean(true).setMsg("更新成功");
        } catch (ImppBusiException imppException) {

            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

}
