package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.busi.bu3;

import cn.estsh.i3plus.ext.mes.api.busi.bu3.ISxPartExtService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesPartExt;
import cn.estsh.i3plus.ext.mes.pojo.repository.MesPartExtRepository;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesPart;
import cn.estsh.i3plus.pojo.mes.bean.MesPartCategory;
import cn.estsh.i3plus.pojo.mes.bean.MesPartProduceType;
import cn.estsh.i3plus.pojo.mes.repository.MesPartCategoryRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesPartProduceTypeRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Slf4j
@Service
public class SxPartExtService implements ISxPartExtService {

    @Autowired
    private MesPartExtRepository partExtRepository;

    @Autowired
    private MesPartCategoryRepository partCategoryRepository;

    @Autowired
    private MesPartProduceTypeRepository partProduceTypeRepository;

    @Override
    public MesPartExt getPartExtByPartNo(String organizeCode, String partNo) {
        return partExtRepository.getByProperty(
                new String[]{MesExtConstWords.ORGANIZE_CODE, MesExtConstWords.IS_DELETED, MesExtConstWords.IS_VALID, MesExtConstWords.PART_NO},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue(), partNo});
    }

    @Override
    public List<MesPartExt> queryPartExtListByModifyDatetime(MesPart part) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(part.getOrganizeCode());
        DdlPreparedPack.timeBuilder(part.getModifyDateTimeStart(), part.getModifyDateTimeEnd(), MesExtConstWords.MODIFY_DATE_TIME, packBean, true);
        return partExtRepository.findByHqlWhere(packBean);
    }

    @Override
    public List<MesPartCategory> queryPartCategoryList(String organizeCode) {
        return partCategoryRepository.findByProperty(
                new String[]{MesExtConstWords.ORGANIZE_CODE, MesExtConstWords.IS_DELETED, MesExtConstWords.IS_VALID},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue()});
    }

    @Override
    public List<MesPartProduceType> queryPartProduceTypeList(String organizeCode) {
        return partProduceTypeRepository.findByProperty(
                new String[]{MesExtConstWords.ORGANIZE_CODE, MesExtConstWords.IS_DELETED, MesExtConstWords.IS_VALID},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue()});
    }

    @Override
    public List<MesPartCategory> queryPartCategoryListByType(String organizeCode, String categoryType) {
        return partCategoryRepository.findByProperty(
                new String[]{MesExtConstWords.ORGANIZE_CODE, MesExtConstWords.IS_DELETED, MesExtConstWords.IS_VALID, MesExtConstWords.CATEGORY_TYPE},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue(), categoryType});
    }

    @Override
    public Map<String, String> queryPartCategoryMapByType(String organizeCode, String categoryType) {
        List<MesPartCategory> partCategoryList = queryPartCategoryListByType(organizeCode, categoryType);
        return CollectionUtils.isEmpty(partCategoryList) ? null : partCategoryList.stream().collect(Collectors.toMap(MesPartCategory::getCategoryCode, MesPartCategory::getCategoryName));
    }

}
