package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.excel;

import cn.estsh.i3plus.ext.mes.api.base.IExcelImportExtService;
import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base.CommonExtService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesProjectConfirm;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportErrorExtModel;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportResultExtModel;
import cn.estsh.i3plus.ext.mes.pojo.repository.MesProjectConfirmRepository;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtEnumUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description : 工程巡回确认项目导入，支持编辑
 * @Reference :
 * @Author : wangjie
 * @CreateDate : 2019-04-23
 * @Modify:
 **/
@Service
public class ProjectConfirmExcelService implements IExcelImportExtService {

    @Autowired
    private MesProjectConfirmRepository projectConfirmRepository;

    @Autowired
    private CommonExtService commonExtService;

    /**
     * 工程巡回确认项目导入
     * @param workbook
     * @param organizeCode
     * @param userName
     * @return
     */
    @Override
    public ExcelImportResultExtModel insertDataByExcel(Workbook workbook, String organizeCode, String userName) {
        //读取表格
        ExcelImportResultExtModel excelImportResultExtModel = this.sheetExtractMesProjectConfirm(workbook.getSheetAt(0), organizeCode, userName);
        //数据入库
        this.insertExcelMesProjectConfirm(excelImportResultExtModel);
        return excelImportResultExtModel;
    }

    /**
     * 工程巡回确认项目导入-读取表格
     * @param sheetAt
     * @param organizeCode
     * @param userName
     * @return
     */
    private ExcelImportResultExtModel sheetExtractMesProjectConfirm(Sheet sheetAt, String organizeCode, String userName) {
        //从0行开始读取
        int totalNumberOfRows = sheetAt.getLastRowNum() + 1;

        //MesProjectConfirm集合
        List<MesProjectConfirm> projectConfirmList = new ArrayList<>();

        //成功数量
        Integer successRowNum = 0;
        //失败数量
        Integer failRowNum = 0;
        //错误的行号
        String errorRows = "";

        //错误行信息集合
        List<ExcelImportErrorExtModel> excelImportErrorExtModels = new ArrayList<>();

        //用于EXCEL表中的项目分类+项目代码唯一校验，key：prodCfgTypeClassify&prodCfgTypeCode，value：row
        Map<String, String> existMap = new HashMap<>();

        //查询工程巡回确认项目表
        List<MesProjectConfirm> projectConfirms = projectConfirmRepository.findByProperty(
                new String[]{MesExtConstWords.ORGANIZE_CODE, MesExtConstWords.IS_DELETED},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()});
        Map<String, List<MesProjectConfirm>> projectConfirmMap = CollectionUtils.isEmpty(projectConfirms) ? null :
                projectConfirms.stream().filter(o -> null != o).collect(Collectors.groupingBy(o -> o.getProdCfgTypeClassify() + "&" + o.getProdCfgTypeCode()));

        //从excel表的第5行数据开始导入，getFirstRowNum是从0行开始读取
        for (int i = (sheetAt.getFirstRowNum() + 4); i < totalNumberOfRows; i ++) {
            Row row = sheetAt.getRow(i);
            //空行跳过
            if (null == row) {
                continue;
            }
            //获取总列数
            Short lastCellNum = row.getLastCellNum();
            if (lastCellNum > 0) {

                int rowNum = i + 1;    //当前行号
                int errorNum = 0;      //错误数量
                String cellNum = "";   //错误列号
                String errorInfo = ""; //错误信息

                //项目分类
                try {
                    //第一列必须有值，否则跳过
                    row.getCell(0).setCellType(CellType.STRING);
                } catch (Exception e) {
                    continue;
                }
                String prodCfgTypeClassify = row.getCell(0, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                //项目代码
                String prodCfgTypeCode = null;
                try {
                    row.getCell(1).setCellType(CellType.STRING);
                    prodCfgTypeCode = row.getCell(1, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }
                //项目名称
                String prodCfgTypeName = null;
                try {
                    row.getCell(2).setCellType(CellType.STRING);
                    prodCfgTypeName = row.getCell(2, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }

                //校验项目分类
                if (StringUtils.isEmpty(MesExtEnumUtil.PROD_CFG_TYPE_CLASSIFY.descriptionOfValue(prodCfgTypeClassify))) {
                    ++errorNum;
                    cellNum = cellNum + "A;";
                    errorInfo = errorInfo + "第A列数据无效;";
                }

                //校验项目代码
                if (StringUtils.isEmpty(prodCfgTypeCode)) {
                    errorNum ++;
                    cellNum += "B;";
                    errorInfo += "第B列数据必填;";
                }

                boolean isExist = false;
                //校验项目分类+项目代码不重复
                String key = MesExtEnumUtil.PROD_CFG_TYPE_CLASSIFY.descriptionOfValue(prodCfgTypeClassify) + "&" + prodCfgTypeCode;
                if (!existMap.containsKey(key)) {
                    existMap.put(key, String.valueOf(rowNum));
                    //校验 项目分类+项目代码 是否已经存在数据库
                    if (StringUtils.isEmpty(errorInfo) && !CollectionUtils.isEmpty(projectConfirmMap) && projectConfirmMap.containsKey(key)) {
                        isExist = true;
                    }
                } else {
                    errorNum ++;
                    cellNum += "A+B;";
                    errorInfo += "第A+B列数据跟第" + existMap.get(key) + "行的第A+B列数据重复;";
                }

                //校验项目名称
                if (StringUtils.isEmpty(prodCfgTypeName)) {
                    errorNum ++;
                    cellNum += "C;";
                    errorInfo += "第C列数据必填;";
                }

                //没有错误的时候，封装MesProjectConfirm
                if (errorNum == 0) {
                    MesProjectConfirm projectConfirm;
                    if(isExist){
                        projectConfirm = projectConfirmMap.get(key).get(0);
                        ConvertBean.serviceModelUpdate(projectConfirm, userName);
                    }else{
                        projectConfirm = new MesProjectConfirm();
                        ConvertBean.serviceModelInitialize(projectConfirm, userName);
                    }
                    projectConfirm.setProdCfgTypeClassify(MesExtEnumUtil.PROD_CFG_TYPE_CLASSIFY.descriptionOfValue(prodCfgTypeClassify));
                    projectConfirm.setProdCfgTypeCode(prodCfgTypeCode);
                    projectConfirm.setProdCfgTypeName(prodCfgTypeName);
                    projectConfirm.setOrganizeCode(organizeCode);

                    projectConfirmList.add(projectConfirm);

                    successRowNum ++;
                } else {
                    //封装错误行信息ExcelImportErrorExtModel
                    excelImportErrorExtModels = commonExtService.getExcelImportErrorModels(excelImportErrorExtModels, rowNum, errorNum, cellNum, errorInfo);

                    errorRows += rowNum + ";";
                    failRowNum ++;
                }
            }
        }

        //校验EXCEL数据
        commonExtService.checkExcelData(failRowNum, successRowNum, errorRows);

        //封装返回结果
        ExcelImportResultExtModel excelImportResultModel = commonExtService.getExcelImportResultModel(failRowNum, successRowNum, excelImportErrorExtModels, errorRows);
        excelImportResultModel.setExcelList((failRowNum > 0) ? null : projectConfirmList);
        return excelImportResultModel;
    }

    /**
     * 工程巡回确认项目扩展导入-数据入库
     * @param excelImportResultExtModel
     */
    private void insertExcelMesProjectConfirm(ExcelImportResultExtModel excelImportResultExtModel) {
        //导入数据
        if (null != excelImportResultExtModel) {
            List<MesProjectConfirm> excelList = excelImportResultExtModel.getExcelList();
            if (!CollectionUtils.isEmpty(excelList)) {
                for (MesProjectConfirm projectConfirm : excelList) {
                    if(StringUtils.isEmpty(projectConfirm.getId())){
                        projectConfirmRepository.insert(projectConfirm);
                    }else{
                        projectConfirmRepository.save(projectConfirm);
                    }
                }
            }
        }
    }
}
